#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*
* 2018/05/27 changes made by Group "parcourPlatformer"
*/

#ifdef GL_ES
precision mediump float;
#endif

float noise (in vec2 _st);
in VertexData {
	vec3 position_world;
	vec3 position_shadow;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;
uniform int mode;
uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;

layout(binding=0) uniform sampler2D textureDiffuse0;
layout(binding=1) uniform sampler2D textureSpecular0;
layout(binding=2) uniform sampler2D frameTex;

layout(binding=4) uniform sampler2DShadow shadowMap;
layout(binding=3) uniform sampler2DShadow lightMap;


vec2 poissonDisk[16] = vec2[](
   vec2( -0.94201624, -0.39906216 ), 
   vec2( 0.94558609, -0.76890725 ), 
   vec2( -0.094184101, -0.92938870 ), 
   vec2( 0.34495938, 0.29387760 ), 
   vec2( -0.91588581, 0.45771432 ), 
   vec2( -0.81544232, -0.87912464 ), 
   vec2( -0.38277543, 0.27676845 ), 
   vec2( 0.97484398, 0.75648379 ), 
   vec2( 0.44323325, -0.97511554 ), 
   vec2( 0.53742981, -0.47373420 ), 
   vec2( -0.26496911, -0.41893023 ), 
   vec2( 0.79197514, 0.19090188 ), 
   vec2( -0.24188840, 0.99706507 ), 
   vec2( -0.81409955, 0.91437590 ), 
   vec2( 0.19984126, 0.78641367 ), 
   vec2( 0.14383161, -0.14100790 ) 
);

uniform float brightness;


uniform vec2 u_resolution;
uniform float u_time;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

// from ShadowMapping tutorial @ opengl-tutorials
float random(vec3 seed, int i) {
	vec4 seed4 = vec4(seed,i);
	float dot_product = dot(seed4, vec4(12.9898,78.233,45.164,94.673));
	return fract(sin(dot_product) * 43758.5453);
}

float shadowCalculation(vec3 norm, vec3 ldir, vec3 fragPosShadow) {
	vec3 n = normalize(norm);
	vec3 l = normalize(ldir);

	float cosTheta = clamp(dot(n, l), 0.0f, 1.0f);
	float visibility = 1.0f;

	float bias = clamp(0.005f * tan(acos(cosTheta)), 0.0f, 0.01f);

	for (int i = 0; i < 4; i++) {
		//  - A random sample, based on the pixel's position in world space.
		//    The position is rounded to the millimeter to avoid too much aliasing
		int index = int(16.0*random(floor(vert.position_world.xyz*1000.0), i))%16;
		visibility -= 0.2f*(1.0f-texture( shadowMap, vec3(fragPosShadow.xy + poissonDisk[index]/700.0f, fragPosShadow.z - bias) ));
		visibility -= 0.2f*(1.0f-texture( lightMap, vec3(fragPosShadow.xy + poissonDisk[index]/700.0f, fragPosShadow.z - bias) ));
	}

	return visibility;
}

void main() {	
	float visibility = shadowCalculation(vert.normal_world, dirL.direction, vert.position_shadow);

	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	color = vec4(1.0f,1.0f,1.0f,1.0f);
	switch(mode){
		case 0:
			vec3 texColor = texture(textureDiffuse0, vert.uv).rgb;
			vec3 specColor = texture(textureSpecular0, vert.uv).rgb;
			color = vec4(texColor * materialCoefficients.x, 1); // ambient
			// add directional light contribution
			color.rgb += visibility * phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color * specColor, materialCoefficients.z, specularAlpha, false, vec3(0));
			// add point light contribution
			color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color * specColor, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
			break;

		case 1:
			color = vec4(1.0f,1.0f,1.0f,1.0f);
			float n = noise(vert.uv*10);
			color =vec4(vec3(n),1.0);
			color = mix(vec4(0.53f,0.81f,0.98f,1.0f), vec4(1.0f),color);
			break;
		case 2:
			vec3 videoColor = texture(frameTex, vert.uv).rgb;
			color = vec4(videoColor * materialCoefficients.x, 1);
			break;
		default:
		    color = vec4(1.0, 1.0, 0.0, 1.0);
			break;
	}
	color *= brightness;
}

float random (in vec2 _st) {
    return fract(sin(dot(_st.xy,
                         vec2(12.9898,78.233)))*
        43758.5453123);
}

float noise (in vec2 _st) {
    vec2 i = floor(_st);
    vec2 f = fract(_st);

    // Four corners in 2D of a tile
    float a = random(i);
    float b = random(i + vec2(1.0, 0.0));
    float c = random(i + vec2(0.0, 1.0));
    float d = random(i + vec2(1.0, 1.0));

    vec2 u = f * f * (3.0 - 2.0 * f);

    return mix(a, b, u.x) +
            (c - a)* u.y * (1.0 - u.x) +
            (d - b) * u.x * u.y;
}
